#include "CalcThread.h"
#include <math.h>

float sinusApproximation(float);
jlong fak(int);

JNIEXPORT void JNICALL Java_CalcThread_calcArray (JNIEnv * env,
                                                  jobject object,
                                                  jintArray returnvals,
                                                  jfloatArray wavevals,
                                                  jint x,
                                                  jint y,
                                                  jfloat t,
                                                  jboolean _calculate_with_sinus,
                                                  jboolean _calculate_with_pointers,
																									jboolean _calculate_with_copy){
  int i = 0;
  int j = 0;
	int h = 0;
  int base = 0;
  float akt = 0;
  float val = 0;
  float time = t;
  int wc = 0;
  long color = 0;
  int wavevals_length,returnvals_length,lines_to_calculate;
  jint* preturns,*pfirstline;
  
  jint *returns = (*env)->GetIntArrayElements(env,returnvals,0);
  jfloat *waves = (*env)->GetFloatArrayElements(env,wavevals,0);
  
  // get array length
  returnvals_length = (*env)->GetArrayLength(env,returnvals);
  wavevals_length = (*env)->GetArrayLength(env,wavevals);
  
  wc = wavevals_length/5;
  
	preturns = &returns[0]; // pointer to first element
	
	if(_calculate_with_copy){
	  lines_to_calculate = 1;
	}else{
	  lines_to_calculate = y;
	}
	
	// for each line
	for(h=0;h<lines_to_calculate;h++) {
  	
  	for (i=0; i<x; i++){    //for all pixels in 1st line do
    	val = 0;
    	for (j=0; j<wavevals_length; j+=5){   //for all waves do
      	akt = (waves[(j+0)] * i/waves[(j+3)]) - (waves[(j+1)] * time/waves[(j+3)]) + waves[(j+2)];

      	akt = fmod(akt,(2*M_PI));

      	// normal sinus, or approx.
      	if(_calculate_with_sinus) { 
				  akt =(float) sin((double) akt); 
			  }
      	else {
        	akt = sinusApproximation(akt);
      	}

      	akt = (akt+1)/2;
      	akt = akt * waves[j+4]; //Amplitude between 0 and 1

      	val += akt;
    	}

    	color = (2 * 255 * (val/wc) );

    	if(_calculate_with_pointers) {
      	if(color < 255){
        	*preturns++ = 0;
        	*preturns++ = (int) (color);
        	*preturns++ = 0;
      	}else{
        	*preturns++ = 0;
        	*preturns++ = (int) (255);
        	*preturns++ = (int) (color-255);
      	}
    	}else{
      	base = (3 * i) + (h * x * 3);
      	if(color < 255){
        	returns[base+0]= 0;
        	returns[base+1]= (int) (color);
        	returns[base+2]= 0;
      	}else{
        	returns[base+0]= 0;
        	returns[base+1]= (int) (255);
        	returns[base+2]= (int) (color-255);
      	}
    	}
  	}
  }

  if(_calculate_with_copy) {
  	//copy first line
  	for(j=1; j<y; j++) {
    	pfirstline = &returns[0];
    	for(i=0;i<x; i++) {
      	if(_calculate_with_pointers) {
        	*preturns++ = *pfirstline++;
        	*preturns++ = *pfirstline++;
        	*preturns++ = *pfirstline++;
      	}else{
        	base = 3*j*x + 3 * i;

        	returns[base+0]= returns[3*i+0];
        	returns[base+1]= returns[3*i+1];
        	returns[base+2]= returns[3*i+2];
      	}
    	}
  	}
	}

 (*env)->ReleaseIntArrayElements(env,returnvals, returns, 0);
 (*env)->ReleaseFloatArrayElements(env,wavevals, waves, 0);
}

/* taylor approximation of sinus */
float sinusApproximation(float akt){
  int negate = 0;
    
    if(akt > M_PI) { 
      akt -= (float)M_PI;
      negate = 1;
    }
    akt = (akt - (float)pow(akt,3.0)/(float)fak(3)
               + (float)pow(akt,5.0)/(float)fak(5) 
               - (float)pow(akt,7.0)/(float)fak(7)
               + (float)pow(akt,9.0)/(float)fak(9)
               - (float)pow(akt,11.0)/(float)fak(11)
               + (float)pow(akt,13.0)/(float)fak(13)
               - (float)pow(akt,15.0)/(float)fak(15)
               + (float)pow(akt,17.0)/(float)fak(17));

     if(negate) { akt *= -1; }
     return(akt);               

}

jlong fak(int x){
  int i;
  jlong fak=1;
  for(i=x; i>0; i--){
    fak *= i;
  }
  return(fak);
}
