import java.awt.*;
import java.awt.geom.*;

/**
 * <b>Programm:</b> WaveJNI<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert, Milan Altenburg<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> holds data for a monochromatic wave
 */

public class MonoWave{
  
  static final float SCALE = 50.0f;
  static final float PHASE = 0.0f;
  
  float _Amplitude;
  float _frequency;
  float _phi;
  float _lambda;
  float _scale;
  boolean _enabled;
  MonoWaveSlider _mwslider = null;

  /**
   * Constructor
   */
  public MonoWave(float Amp,float freq,float lambda){
    //set up defaults:
    _mwslider = new MonoWaveSlider(this);
    _mwslider.getAmplitudeSlider().setFloatMinimum(0.0f);
    _mwslider.getAmplitudeSlider().setFloatMaximum(20.0f);
    _mwslider.getFrequencySlider().setFloatMinimum(-80.0f);
    _mwslider.getFrequencySlider().setFloatMaximum(80.0f);
    _mwslider.getWaveLengthSlider().setFloatMinimum(0.0f);
    _mwslider.getWaveLengthSlider().setFloatMaximum(20.0f);
    _mwslider.getPhaseSlider().setFloatMinimum(-2.0f*(float)Math.PI);
    _mwslider.getPhaseSlider().setFloatMaximum( 2.0f*(float)Math.PI);

    setAmplitude(Amp);
    setFrequency(freq);
    setWaveLength(lambda);
    setPhase(0.0f);
    setScale(SCALE);
    enable(true);
  }

  /**
   * enables/disables the wave
   */
  public void enable(boolean enabled){
    _enabled = enabled;
  }

  /**
   * returns the on/off state of the wave
   */
  public boolean is_enabled(){
    return _enabled;
  }

  /**
   * Return MonoWaveSliderobject
   */
  public MonoWaveSlider getMonoWaveSlider(){
    return _mwslider;
  }

  /**
   * sets amplitude
   */
  public void setAmplitude(float A) {
    _Amplitude = A;
    _mwslider.getAmplitudeSlider().setFloatValue(A);
  }

  /**
   * sets wavelength
   */
  public void setWaveLength(float lambda) {
    _lambda = lambda;
    _mwslider.getWaveLengthSlider().setFloatValue(lambda);
  }

  /**
   * sets phaseshift
   */
  public void setPhase(float phi) {
    _phi = phi;
    _mwslider.getPhaseSlider().setFloatValue(phi);
  }

  /**
   * sets frequency
   */
  public void setFrequency(float f) {
    _frequency = f;
    _mwslider.getFrequencySlider().setFloatValue(f);
  }

  /**
   * sets scale
   */
  public void setScale(float scale) {
    _scale = scale;
  }

  /**
   * gets amplitude
   */
  public float getAmplitude() {
    return(_Amplitude);
  }

  /**
   * gets wavelength
   */
  public float getWaveLength() {
    return (_lambda);
  }

  /**
   * gets phaseshift
   */
  public float getPhase() {
    return (_phi);
  }

  /**
   * gets frequency
   */
  public float getFrequency() {
    return(_frequency);
  }

  /**
   * gets circlefrequency (omega)
   */
  public float getCircleFrequency(){
    return (2*(float)Math.PI*_frequency);
  }

  /**
   * gets Wavekey (k)
   */
  public float getWaveKey(){
    return (2*(float)Math.PI/_lambda);
  }

  /**
   * gets scale
   */
  public float getScale() {
    return(_scale);
  }

  /**
   * gets ratio between SliderMaximum and Amplitude
   */
  public float getRatio() {
    return(_Amplitude/_mwslider.getAmplitudeSlider().getFloatMaximum());
  }

}
