import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

/**
 * <b>Programm:</b> WaveThread<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert, Milan Altenburg<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> displays the animation
 */
public class Painting extends Canvas{
  
  static final float FPS_MULTIPLIER = 1000.0f;
  static final int TYPE_INT_RGB = 1; // for BufferedImage 
  
  int _width = 0;
  int _height= 0;
  long _turn = 0;
  long _starttime = 0;
  CalcThread _calcthread = null;
  BufferedImage _rawbuffer  = null;
  BufferedImage _outbuffer = null;
  AffineTransform  _at = null;
  AffineTransformOp _atop = null;
  

  /**
   * Constructor
   */
  public Painting(CalcThread cl) {
    _calcthread = cl;
    _at = new AffineTransform();
    _atop = new AffineTransformOp(_at,AffineTransformOp.TYPE_BILINEAR);
  }

  /**
   * Set zoom
   */
  public void setScale(double zoom){
    _at.setToScale(zoom,zoom);
    _atop = new AffineTransformOp(_at,AffineTransformOp.TYPE_BILINEAR);
  }

  /**
   * offscreen rendering
   */
  public void offpaint(){
    //set data from _calcthread:
    _rawbuffer.setData(_calcthread.getRaster());

    //apply scale
    _atop.filter(_rawbuffer,_outbuffer);

    // draw FPS
    Graphics2D g2d = (Graphics2D) _outbuffer.getGraphics();
    g2d.setPaint(Color.white);
    float fps = FPS_MULTIPLIER*(float)_turn / 
    				(float)(System.currentTimeMillis()-_starttime);
    g2d.drawString(fps+"fps",10,10);
  }

  /**
   * Overwrites original update and removes the screnn clearing as we use
   * doublebuffering by using offpaint()
   */
  public void update(Graphics g){
    paint(g);
  }


  /**
   * reset FPS counter
   */
  public void resetFPS(){
    _starttime = System.currentTimeMillis();
    _turn = 1;
  }

  /**
   * initializes the offscreenbuffer, calls offpaint() and paints the buffer
   */
  public void paint(Graphics g){
    // if resized, inform _calcthread, create new BufferedImages
    if( (_width != getWidth()) || (_height != getHeight()) ) {
      _width  = getWidth(); // new values
      _height = getHeight();
      _rawbuffer = new BufferedImage(_width,_height, TYPE_INT_RGB );
      _outbuffer = new BufferedImage(_width,_height, TYPE_INT_RGB );
      resetFPS();
      // set new values in _calcthread, same object
      _calcthread.setXY(_width,_height);
    }else {
      _turn++;
      // avoid overflow:
      if(_turn == 75) {
        float fps = FPS_MULTIPLIER*(float)_turn / 
        				(float)(System.currentTimeMillis()-_starttime);
        //print performance log to STDOUT
        System.out.println(_width+"x"+_height+"\t"+_calcthread.getOptions()+"\t"+fps);
        resetFPS();
      }
      offpaint(); //renew global offscreenbuffer
      g.drawImage(_outbuffer,0,0,this);
    }
  }
}
