import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

/**
 * <b>Programm:</b> WaveJNI<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert, Milan Altenburg<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> Main class constructs GUI Components and Threads
 */
public class WaveJNIMain extends ManagedPanel
                         implements ActionListener,
                                    ChangeListener
{
  static final float DEFAULT_AMPLITUDE = 20.0f;
  static final int WAVE_COUNT = 5;
  
  MonoWave[] _wave_array = new MonoWave[WAVE_COUNT];
  Painting _paint_area  = null;
  PullThread _pullthread = null;
  CalcThread _calcthread = null;
  JCheckBox _usejava = null;
  JCheckBox _usepointers =null;

  /**
   * Constructor
   */
  public WaveJNIMain() {
    _wave_array[0] = new MonoWave((float)DEFAULT_AMPLITUDE,
    											(float)10,(float)2.0);
    _wave_array[1] = new MonoWave((float)DEFAULT_AMPLITUDE,
    											(float)20,(float)1.0);
    _wave_array[2] = new MonoWave((float)DEFAULT_AMPLITUDE,
    											(float)40,(float)0.5);
    _wave_array[3] = new MonoWave((float)DEFAULT_AMPLITUDE,
    											(float)10,(float)2.0);
    _wave_array[4] = new MonoWave((float)DEFAULT_AMPLITUDE,
    											(float)10,(float)2.0);

    /* CalcThread with Data, no extra RasterFifo! */
    /* calcthread is started when method setXY invoked */
    _calcthread = new CalcThread(_wave_array,Color.blue);

    /* in Painting calcthread.start() is invoked */
    _paint_area = new Painting(_calcthread);

    _pullthread = new PullThread(_paint_area, _calcthread);

    this.addComponent(_paint_area,1,1,5,1,2,2);

    this.addComponent(new JLabel("frequency") ,2,2,1,1,1,0);
    this.addComponent(new JLabel("wavelength"),3,2,1,1,1,0);
    this.addComponent(new JLabel("phaseshift"),4,2,1,1,1,0);
    this.addComponent(new JLabel("amplitude"),5,2,1,1,1,0);

    for (int i=0; i<5; i++){
      MonoWaveSlider _wave_slider = _wave_array[i].getMonoWaveSlider();

      this.addComponent(_wave_slider.getEnableCheckbox()  ,1,i+3,1,1,1,0);
      this.addComponent(_wave_slider.getFrequencySlider() ,2,i+3,1,1,1,0);
      this.addComponent(_wave_slider.getWaveLengthSlider(),3,i+3,1,1,1,0);
      this.addComponent(_wave_slider.getPhaseSlider()     ,4,i+3,1,1,1,0);
      this.addComponent(_wave_slider.getAmplitudeSlider() ,5,i+3,1,1,1,0);
    }

    _usejava = new JCheckBox("Java",true);
    _usejava.addActionListener(this);
    _usejava.setName("usejava");
    this.addComponent(_usejava,1,9,1,1,1,0);

    _usepointers = new JCheckBox("Pointer",false);
    _usepointers.addActionListener(this);
    _usepointers.setName("usepointers");
    _usepointers.setEnabled(false);
    this.addComponent(_usepointers,2,9,1,1,1,0);

    JCheckBox usesin = new JCheckBox("use sin",true);
    usesin.addActionListener(this);
    usesin.setName("usesin");
    this.addComponent(usesin,3,9,1,1,1,0);

    JCheckBox usecopy = new JCheckBox("use copy",false);
    usecopy.addActionListener(this);
    usecopy.setName("usecopy");
    this.addComponent(usecopy,4,9,1,1,1,0);


    FloatJSlider zoom = new FloatJSlider(1.0f,5.0f,1.0f);
    zoom.addChangeListener(this);
    this.addComponent(zoom,5,9,1,1,1,0);

  }

  /**
   * Eventhandler for Checkboxes
   */
  public void actionPerformed(ActionEvent _action_event){
    String name = ((JCheckBox) _action_event.getSource()).getName();
    if(name == "usejava"){
      if(_usejava.isSelected()){
        _usepointers.setEnabled(false);
      }else{
        _usepointers.setEnabled(true);
      }
      _calcthread.switchCalcMethod();
    }else if(name == "usesin"){
      _calcthread.switchSinusCalcMethod();
    }else if(name == "usepointers"){
      _calcthread.switchUsageOfPointers();
    }else if(name == "usecopy"){
      _calcthread.switchUsageOfCopy();
    }
		// trigger reset of fps calculation
    _paint_area.resetFPS();
  }

  /**
   * Eventhandler for zoom slider
   */
  public void stateChanged (ChangeEvent _change_event){
    FloatJSlider zoom = (FloatJSlider) _change_event.getSource();
    _paint_area.setScale((double) zoom.getFloatValue());
    zoom.setToolTipText(Float.toString(zoom.getFloatValue()));
  }
}


